@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (C) 2015 J.F.Dockes
+# Copyright (C) 2015-2020 J.F.Dockes
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
 #   the Free Software Foundation; either version 2 of the License, or
@@ -13,17 +13,26 @@
 #   You should have received a copy of the GNU General Public License
 #   along with this program; if not, write to the
 #   Free Software Foundation, Inc.,
-#   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 ######################################
-from __future__ import print_function
+
+# Note that .docx and .xlsx are now normally processed by the C++ mh_xslt.cpp
+# module. See the openxml-xxx.xsl files for the style sheets used by the C++.
+# 
+# .pptx and .vsdx are processed by this Python module because the C++ module
+# can't process their multiple document structure (pages) at the moment.
 
 import sys
 from zipfile import ZipFile
-import fnmatch
 import rclexecm
 from rclbasehandler import RclBaseHandler
 import rclxslt
+import re
+import os
 
+#
+# Common style sheet for the openxml metadata
+#
 meta_stylesheet = '''<?xml version="1.0"?>
 <xsl:stylesheet 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
@@ -75,6 +84,9 @@ meta_stylesheet = '''<?xml version="1.0"?>
 </xsl:stylesheet>
 '''
 
+#####################################
+# .docx definitions. Not used any more by Recoll in its default config
+
 word_tagmatch = 'w:p'
 word_xmlns_decls = '''xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
 xmlns:ve="http://schemas.openxmlformats.org/markup-compatibility/2006"
@@ -90,12 +102,19 @@ xmlns:wne="http://schemas.microsoft.com/office/word/2006/wordml"
 word_moretemplates = ''
 
 
+#####################################
+# .xlsx definitions. Not used any more by Recoll in its default config
+
 xl_tagmatch = 'x:t'
 xl_xmlns_decls='''xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
 xmlns:x="http://schemas.openxmlformats.org/spreadsheetml/2006/main"
  '''
 xl_moretemplates = ''
 
+
+#####################
+# .pptx definitions
+
 pp_tagmatch = 'a:t'
 pp_xmlns_decls = '''xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
 xmlns:a="http://schemas.openxmlformats.org/drawingml/2006/main" 
@@ -108,6 +127,21 @@ pp_moretemplates = '''<xsl:template match="p:attrName">
 </xsl:template>
 '''
 
+
+#####################
+# .vsdx definitions
+
+vs_tagmatch = 'Text'
+vs_xmlns_decls = '''xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
+xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships"
+'''
+vs_moretemplates = ''
+
+
+##############################
+# Common style sheet (with replaceable parts) for .pptx and .vsdx (also .docx
+# and .xlsx, but not used by default).
+
 content_stylesheet = '''<?xml version="1.0"?>
 <xsl:stylesheet @XMLNS_DECLS@ >
 
@@ -133,7 +167,7 @@ content_stylesheet = '''<?xml version="1.0"?>
 class OXExtractor(RclBaseHandler):
     def __init__(self, em):
         super(OXExtractor, self).__init__(em)
-
+        self.forpreview = os.environ.get("RECOLL_FILTER_FORPREVIEW", "no")
 
     # Replace values inside data style sheet, depending on type of doc
     def computestylesheet(self, nm):
@@ -179,16 +213,41 @@ class OXExtractor(RclBaseHandler):
             pass
 
         try:
-            stl = self.computestylesheet('pp')
-            # Note that we'd need a numeric sort really (else we get slide1
-            # slide11 slide2)
-            for fn in sorted(zip.namelist()):
-                if fnmatch.fnmatch(fn, 'ppt/slides/slide*.xml'):
+            stl = None
+            # Extract number suffix for numeric sort
+            for prefix in ("ppt/slides/slide","ppt/notesSlides/notesSlide"):
+                exp = prefix + '[0-9]+' + '.xml'
+                names = [fn for fn in zip.namelist() if re.match(exp, fn)]
+                for fn in sorted(
+                        names,
+                        key=lambda e,prefix=prefix: int(e[len(prefix):len(e)-4])):
+                    if stl is None:
+                        stl = self.computestylesheet('pp')
                     content = zip.read(fn)
+                    if self.forpreview == "yes":
+                        docdata += b"<h2>" + fn.encode("utf-8",errors="ignore") + b"</h2>"
                     docdata += rclxslt.apply_sheet_data(stl, content)
-        except:
+        except Exception as ex:
+            #self.em.rclog("PPT Exception: %s" % ex)
             pass
 
+        try:
+            stl = None
+            # Extract number suffix for numeric sort
+            prefix = 'visio/pages/page'
+            exp = prefix + '[0-9]+' + '.xml'
+            names = [fn for fn in zip.namelist() if re.match(exp, fn)]
+            for fn in sorted(
+                    names,
+                    key=lambda e,prefix=prefix: int(e[len(prefix):len(e)-4])):
+                if stl is None:
+                    stl = self.computestylesheet('vs')
+                content = zip.read(fn)
+                docdata += rclxslt.apply_sheet_data(stl, content)
+        except Exception as ex:
+            #self.em.rclog("VISIO Exception: %s" % ex)
+            pass
+        
         docdata += b'</body></html>'
 
         return docdata
